/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "md5sumfile.h"

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qapplication.h>
#include <qregexp.h>

namespace KDigest {

/**
@brief Default constructor
@param path Path of the SFV file
@param create If true, the SFV file will be created
@param parent parent object (passed to DigestFile and then to QObject)
@param name object's name (passed to DigestFile and then to QObject)
*/
MD5SUMfile::MD5SUMfile(const QString &path, bool create, QObject *parent, const char *name)
	: DigestFile(parent, name)
{
	if ( ! create )
	{
		QFile f(path);
		if ( ! f.open(IO_ReadOnly) ) {
			///@todo Thrown an error
			return;
		}
		
		QTextStream ts(&f);
		
		QRegExp re("([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})[\\* ]{1} (.*)");
		QString line;
		while(!ts.atEnd())
		{
			line = ts.readLine();
			if ( re.search(line) == -1 )
				continue;
			
			uint32_t md5Bytes[4]; bool ok = false;
			md5Bytes[0] = re.cap(1).toUInt(&ok, 16); if ( ! ok ) continue;
			md5Bytes[1] = re.cap(2).toUInt(&ok, 16); if ( ! ok ) continue;
			md5Bytes[2] = re.cap(3).toUInt(&ok, 16); if ( ! ok ) continue;
			md5Bytes[3] = re.cap(4).toUInt(&ok, 16); if ( ! ok ) continue;
			
			MD5SUMentry *e = new MD5SUMentry;
			
			e->filename = re.cap(5);
			e->status = dsNotChecked;

			memcpy(e->md5, md5Bytes, sizeof(e->md5));
			memset(e->aMd5, 0, sizeof(e->aMd5));
			
			m_entries.append(e);
		}
		
		f.close();
	} else {
		///@todo write
	}
}

/**
@brief Check the entries loaded from the SFV file for exactness and update the m_entries list
@param path Path to apply to the entries' filenames to get the complete path
*/
bool MD5SUMfile::testFile(const QString &path)
{
	int i = -1;
	bool allok = true;
	QDir currDir(path);
	for( DigestEntries::iterator it = m_entries.begin(); it != m_entries.end(); it++ )
	{
		MD5SUMentry *curr = reinterpret_cast<MD5SUMentry *>(*it);
		if ( ! currDir.exists( curr->filename ) )
		{
			allok = false;
			curr->status = dsNotFound;
			continue;
		}
		
		QFile chkFile( path + '/' + curr->filename );
		if ( ! chkFile.open(IO_ReadOnly) )
		{
			allok = false;
			curr->status = dsErrorAccess;
			continue;
		}
		
		KMD5 context(NULL, 0);
		context.reset();
		context.update(chkFile);
		qApp->processEvents();
		
		// Doing this because it's much more simpler than a hton with a 128-bit value ;)
		QRegExp re("([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})([0-9a-fA-F]{8})");
		QString digest = context.hexDigest(); re.search(digest);
		uint32_t md5Bytes[4];
		md5Bytes[0] = re.cap(1).toUInt(NULL, 16);
		md5Bytes[1] = re.cap(2).toUInt(NULL, 16);
		md5Bytes[2] = re.cap(3).toUInt(NULL, 16);
		md5Bytes[3] = re.cap(4).toUInt(NULL, 16);
		memcpy(curr->aMd5, md5Bytes, sizeof(curr->aMd5));
		
		if ( memcmp(curr->aMd5, curr->md5, sizeof(curr->md5)) == 0 )
			curr->status = dsOk;
		else
		{
			curr->status = dsBadDigest;
			allok = false;
		}
		emit cycleCompleted(++i);
	}
	emit cycleCompleted(++i);
	
	return allok;
}

MD5SUMfile::~MD5SUMfile()
{
}


};
#include "md5sumfile.moc"
